package org.eclipse.jdt.internal.compiler.problem;

import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.eclipse.jdt.internal.compiler.util.Util;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.CategorizedProblem;
import java.util.Locale;
import org.eclipse.jdt.internal.compiler.util.HashtableOfInt;
import org.eclipse.jdt.internal.compiler.IProblemFactory;

public class DefaultProblemFactory implements IProblemFactory
{
    public HashtableOfInt messageTemplates;
    private Locale locale;
    private static HashtableOfInt DEFAULT_LOCALE_TEMPLATES;
    private static final char[] DOUBLE_QUOTES;
    private static final char[] SINGLE_QUOTE;
    private static final char[] FIRST_ARGUMENT;
    
    static {
        DOUBLE_QUOTES = "''".toCharArray();
        SINGLE_QUOTE = "'".toCharArray();
        FIRST_ARGUMENT = "{0}".toCharArray();
    }
    
    public DefaultProblemFactory() {
        this(Locale.getDefault());
    }
    
    public DefaultProblemFactory(Locale loc) {
        this.setLocale(loc);
    }
    
    public CategorizedProblem createProblem(char[] originatingFileName, int problemId, String[] problemArguments, String[] messageArguments, int severity, int startPosition, int endPosition, int lineNumber, int columnNumber) {
        return new DefaultProblem(originatingFileName, this.getLocalizedMessage(problemId, messageArguments), problemId, problemArguments, severity, startPosition, endPosition, lineNumber, columnNumber);
    }
    
    public CategorizedProblem createProblem(char[] originatingFileName, int problemId, String[] problemArguments, int elaborationId, String[] messageArguments, int severity, int startPosition, int endPosition, int lineNumber, int columnNumber) {
        return new DefaultProblem(originatingFileName, this.getLocalizedMessage(problemId, elaborationId, messageArguments), problemId, problemArguments, severity, startPosition, endPosition, lineNumber, columnNumber);
    }
    
    private static final int keyFromID(int id) {
        return id + 1;
    }
    
    public Locale getLocale() {
        return this.locale;
    }
    
    public void setLocale(Locale locale) {
        if (locale == this.locale) {
            return;
        }
        this.locale = locale;
        if (Locale.getDefault().equals(locale)) {
            if (DefaultProblemFactory.DEFAULT_LOCALE_TEMPLATES == null) {
                DefaultProblemFactory.DEFAULT_LOCALE_TEMPLATES = loadMessageTemplates(locale);
            }
            this.messageTemplates = DefaultProblemFactory.DEFAULT_LOCALE_TEMPLATES;
        }
        else {
            this.messageTemplates = loadMessageTemplates(locale);
        }
        return;
    }
    
    public final String getLocalizedMessage(int id, String[] problemArguments) {
        return this.getLocalizedMessage(id, 0, problemArguments);
    }
    
    public final String getLocalizedMessage(int id, int elaborationId, String[] problemArguments) {
        String rawMessage = (String)this.messageTemplates.get(keyFromID(id & 0xFFFFFF));
        if (rawMessage == null) {
            return "Unable to retrieve the error message for problem id: " + (id & 0xFFFFFF) + ". Check compiler resources.";
        }
        char[] message = rawMessage.toCharArray();
        if (elaborationId != 0) {
            String elaboration = (String)this.messageTemplates.get(keyFromID(elaborationId));
            if (elaboration == null) {
                return "Unable to retrieve the error message elaboration for elaboration id: " + elaborationId + ". Check compiler resources.";
            }
            message = CharOperation.replace(message, DefaultProblemFactory.FIRST_ARGUMENT, elaboration.toCharArray());
        }
        message = CharOperation.replace(message, DefaultProblemFactory.DOUBLE_QUOTES, DefaultProblemFactory.SINGLE_QUOTE);
        if (problemArguments == null) {
            return new String(message);
        }
        int length = message.length;
        int start = 0;
        int end = length;
        StringBuffer output = null;
        if ((id & Integer.MIN_VALUE) != 0x0) {
            output = new StringBuffer(10 + length + problemArguments.length * 20);
            output.append((String)this.messageTemplates.get(keyFromID(514)));
        }
        while ((end = CharOperation.indexOf('{', message, start)) > -1) {
            if (output == null) {
                output = new StringBuffer(length + problemArguments.length * 20);
            }
            output.append(message, start, end - start);
            if ((start = CharOperation.indexOf('}', message, end + 1)) > -1) {
                try {
                    output.append(problemArguments[CharOperation.parseInt(message, end + 1, start - end - 1)]);
                }
                catch (NumberFormatException ex) {
                    output.append(message, end + 1, start - end);
                }
                catch (ArrayIndexOutOfBoundsException ex2) {
                    return "Cannot bind message for problem (id: " + (id & 0xFFFFFF) + ") \"" + new String(message) + "\" with arguments: {" + Util.toString(problemArguments) + "}";
                }
                ++start;
            }
            else {
                output.append(message, end, length);
                return new String(output.toString());
            }
        }
        if (output == null) {
            return new String(message);
        }
        output.append(message, start, length - start);
        return new String(output.toString());
    }
    
    public final String localizedMessage(CategorizedProblem problem) {
        return this.getLocalizedMessage(problem.getID(), problem.getArguments());
    }
    
    public static HashtableOfInt loadMessageTemplates(Locale loc) {
        ResourceBundle bundle = null;
        String bundleName = "org.eclipse.jdt.internal.compiler.problem.messages";
        try {
            bundle = ResourceBundle.getBundle(bundleName, loc);
        }
        catch (MissingResourceException e) {
            System.out.println("Missing resource : " + bundleName.replace('.', '/') + ".properties for locale " + loc);
            throw e;
        }
        HashtableOfInt templates = new HashtableOfInt(700);
        Enumeration<String> keys = bundle.getKeys();
        while (keys.hasMoreElements()) {
            String key = (String)keys.nextElement();
            try {
                int messageID = Integer.parseInt(key);
                templates.put(keyFromID(messageID), bundle.getString(key));
            }
            catch (NumberFormatException ex) {}
            catch (MissingResourceException ex2) {}
        }
        return templates;
    }
}
